# Generated by Django 4.1.2 on 2022-12-26 06:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0002_remove_keywords_type_keywords_description_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='keywords',
            name='avalable',
            field=models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference', max_length=10, null=True),
        ),
    ]
