# Generated by Django 4.1.2 on 2023-01-06 13:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0003_snortruleviewarray_htmlid_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='snortrule',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
    ]
