# Generated by Django 4.1.2 on 2022-12-19 18:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcaps', '0002_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='pcap',
            old_name='rule_to_validate',
            new_name='rule_to_validate2',
        ),
        migrations.RenameField(
            model_name='white_pcap',
            old_name='rule_to_validate',
            new_name='rule_to_validate2',
        ),
    ]
