# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('settings', '0001_initial'),
        ('pcaps', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SnortRule',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('active', models.BooleanField(default=False)),
                ('admin_locked', models.BooleanField(default=False)),
                ('user', models.CharField(blank=True, default=0, max_length=100)),
                ('name', models.CharField(max_length=100)),
                ('type', models.TextField(choices=[('a', 'type a'), ('b', 'Preprocessor')], max_length=30)),
                ('content', models.TextField(max_length=2048)),
                ('description', models.TextField(max_length=256)),
                ('extra', models.TextField(blank=True, max_length=256)),
                ('location', models.CharField(max_length=256)),
                ('request_ref', models.CharField(blank=True, max_length=12)),
                ('main_ref', models.CharField(blank=True, max_length=12)),
                ('date', models.DateTimeField(auto_now=True)),
                ('template', models.TextField(blank=True, max_length=512)),
                ('group', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='settings.attackgroup')),
                ('pcap_legal_check', models.ManyToManyField(blank=True, related_name='pcap_legal_check', to='pcaps.white_pcap')),
                ('pcap_sanity_check', models.ManyToManyField(blank=True, related_name='pcap_sanity_check', to='pcaps.pcap')),
            ],
            options={
                'ordering': ('name', 'type', 'date'),
            },
        ),
    ]
