# Generated by Django 4.1.2 on 2023-01-06 06:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0002_snortruleviewarray'),
    ]

    operations = [
        migrations.AddField(
            model_name='snortruleviewarray',
            name='htmlId',
            field=models.CharField(blank=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='snortruleviewarray',
            name='typeOfItem',
            field=models.CharField(blank=True, max_length=20),
        ),
    ]
