# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('snort', '0001_initial'),
        ('pcaps', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='white_pcap',
            name='rule_to_validate',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='snort.snortrule'),
        ),
        migrations.AddField(
            model_name='pcap',
            name='rule_to_validate',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='snort.snortrule'),
        ),
    ]
