# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Pcap',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(max_length=256)),
                ('extra', models.TextField(blank=True, max_length=256)),
                ('pcap_file', models.FileField(null=True, upload_to='pacp_repo/')),
                ('date', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='white_Pcap',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(max_length=256)),
                ('extra', models.TextField(blank=True, max_length=256)),
                ('pcap_file', models.FileField(null=True, upload_to='pacp_repo/')),
                ('date', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]
