# Generated by Django 4.1.2 on 2023-01-05 14:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SnortRuleViewArray',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('typeOfItem', models.CharField(blank=True, max_length=12)),
                ('locationX', models.IntegerField()),
                ('locationY', models.IntegerField()),
                ('value', models.CharField(blank=True, max_length=256)),
                ('snortId', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='snort.snortrule')),
            ],
        ),
    ]
