# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='attackGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(max_length=256)),
            ],
        ),
        migrations.CreateModel(
            name='keywords',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of site-wide variable', max_length=200)),
                ('type', models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference - must be valid JSON', max_length=100, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Setting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of site-wide variable', max_length=200)),
                ('value', models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference - must be valid JSON', max_length=100, null=True)),
            ],
        ),
    ]
