{% extends 'admin/base.html' %}

{% block content %}
{% load static %}
<p style="color:red;">uploading a csv is an update/insert action<br>
Do it with ATTENTION!!!
  <form method="post" enctype="multipart/form-data">
    {% csrf_token %}
    <input type="file" name="myfile">
    <button type="submit">load csv</button>
  </form>
  <img width="30%" src="/static/admin/image/images.png"/></p>

{% endblock %}